package com.contentsquare.flutter.infrastructure.core.states

/**
 * A class that holds the state of the Native SDK.
 * The state is composed of:
 * * sessionReplayEnabled: true if session replay is enabled, false otherwise.
 * * apiErrorsEnabled: true if api errors are enabled, false otherwise.
 * * crashReporterEnabled: true if crash reporter is enabled, false otherwise.
 * * featureFlags: list of project feature flags.
 */
object ContentsquarePluginState {
    private var sessionReplayEnabled: Boolean = false
    private var apiErrorsEnabled: Boolean = false
    private var crashReporterEnabled: Boolean = false
    private var featureFlags: List<Map<String, Any>> = emptyList()

    fun setSessionReplayState(enable : StateObject<Boolean>) {
        sessionReplayEnabled = enable.value
    }

    fun setApiErrorsState(enable : StateObject<Boolean>) {
        apiErrorsEnabled = enable.value
    }

    fun setCrashReporterState(enable : StateObject<Boolean>) {
        crashReporterEnabled = enable.value
    }

    fun setFeatureFlagsState(featureFlagsState: StateObject<List<Map<String, Any>>>) {
        featureFlags = featureFlagsState.value
    }

    fun getSessionReplayState() : StateObject<Boolean> {
        return StateObject(sessionReplayEnabled)
    }

    fun getApiErrorsState() : StateObject<Boolean> {
        return StateObject(apiErrorsEnabled)
    }

    fun getCrashReporterState() : StateObject<Boolean> {
        return StateObject(crashReporterEnabled)
    }

    fun getFeatureFlagsState() : StateObject<List<Map<String, Any>>> {
        return StateObject(featureFlags)
    }
}

/**
 * A class that represents the type of a state.
 */
class StateObject<T>(val value: T)