import 'package:contentsquare/src/james_dean2936.dart';
import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/alan_arkin2950.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget
    with michael_douglas483 {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _gregory_peck517();
}

class _gregory_peck517 extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _leonardo_dicaprio1529;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final christopher_walken1530 = sidney_poitier518.humphrey_bogart40(
      context,
      martin_sheen2692: true,
    );
    if (christopher_walken1530 != null) {
      _leonardo_dicaprio1529 = widget.maskingConfig.merge(
        christopher_walken1530.john_goodman259,
      );
    } else {
      _leonardo_dicaprio1529 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return sidney_poitier518(
      john_goodman259: _leonardo_dicaprio1529,
      james_stewart295: widget.child,
    );
  }
}


class sidney_poitier518 extends InheritedWidget
    with michael_douglas483 {
  
  final MaskingConfig john_goodman259;

  
  const sidney_poitier518({
    required this.john_goodman259,
    Key? alan_arkin1693,
    required Widget james_stewart295,
  }) : super(key: alan_arkin1693, child: james_stewart295);

  
  static sidney_poitier518? humphrey_bogart40(
    BuildContext kirk_douglas370, {
    required bool martin_sheen2692,
  }) {
    return martin_sheen2692
        ? kirk_douglas370.dependOnInheritedWidgetOfExactType<sidney_poitier518>()
        : kirk_douglas370.matthew_mcconaughey1047<sidney_poitier518>();
  }

  @override
  bool updateShouldNotify(sidney_poitier518 oldWidget) {
    return oldWidget.john_goodman259 != john_goodman259;
  }
}

extension on BuildContext {
  
  T? matthew_mcconaughey1047<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<sidney_poitier518>()
        ?.widget as T?;
  }
}
