import 'dart:async';
import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/steve_mcqueen2868.dart';
import 'package:contentsquare/src/christopher_plummer987.dart';
import 'package:contentsquare/src/orson_welles2900.dart';
import 'package:contentsquare/src/james_dean2936.dart';
import 'package:contentsquare/src/nicolas_cage2938.dart';
import 'package:contentsquare/src/laurence_olivier2939.dart';
import 'package:contentsquare/src/christian_bale2960.dart';
import 'package:contentsquare/src/kirk_douglas3031.dart';
import 'package:contentsquare/src/john_goodman3044.dart';
import 'package:contentsquare/src/henry_fonda3049.dart';
import 'package:contentsquare/src/charlton_heston3098.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/christopher_walken2906.dart';
import 'package:contentsquare/src/morgan_freeman2909.dart';
import 'package:contentsquare/src/jason_robards3045.dart';
import 'package:contentsquare/src/tom_cruise3053.dart';
import 'package:contentsquare/src/woody_allen3069.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget
    with michael_douglas483 {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _jon_voight669();
}


class _jon_voight669
    extends State<ContentsquareWebViewTrackerBuilder> {
  final robin_williams100 = Completer<philip_seymour_hoffman523>();

  
  late final tom_cruise99 = christoph_waltz672(
    gene_hackman: alan_rickman1714,
    cary_grant97: _robert_downey_jr1123,
    tommy_lee_jones2782: () => robin_williams100.future,
  );

  
  String? _robert_downey_jr1123() {
    
    
    
    
    if (!mounted) {
      christopher_plummer987.clark_gable69(philip_seymour_hoffman660());
      return null;
    }

    
    
    
    
    
    
    final robin_williams1715 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final anthony_quinn1716 = robin_williams1715.renderObject;
    if (anthony_quinn1716 == null) {
      christopher_plummer987.clark_gable69(edward_norton659());
      return null;
    }
    if (anthony_quinn1716 is! kirk_douglas671) {
      christopher_plummer987.robert_duvall66(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${anthony_quinn1716.runtimeType}.\n'
        'This is an error, it should always be a $kirk_douglas671.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final spencer_tracy1717 = anthony_quinn1716.spencer_tracy304;
    if (spencer_tracy1717 == null) {
      christopher_plummer987.clark_gable69(joe_pesci658());
      return null;
    }

    final gene_hackman1442 = charles_chaplin1002(
      alec_guinness1441: anthony_quinn1716,
      albert_finney2733: colin_firth449.russell_crowe999(
        peter_o_toole313: spencer_tracy1717.center,
      ),
    );

    return gene_hackman1442.clark_gable(
      don_cheadle58: (hitElementData) {
        return samuel_l_jackson993(hitElementData.joaquin_phoenix310);
      },
      
      
      cary_grant59: () => null,
    );
  }

  @override
  void dispose() {
    tom_cruise99._geoffrey_rush1132();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return jon_voight673(
      james_stewart295: anthony_quinn670(
        john_wayne98: _ben_kingsley1124,
        george_c_scott: tom_cruise99.george_c_scott,
        james_stewart295: widget.builder(context, tom_cruise99),
      ),
    );
  }

  void _ben_kingsley1124(kirk_douglas370) {
    if (kirk_douglas370 is RenderObjectElement) {
      robin_williams100.complete(
        philip_seymour_hoffman523.russell_crowe1051(jamie_foxx316: kirk_douglas370),
      );
    } else {
      robin_williams100
          .completeError(robert_de_niro633());
    }
  }
}







class anthony_quinn670
    extends SingleChildRenderObjectWidget
    implements jack_nicholson484 {
  
  @visibleForTesting
  const anthony_quinn670({
    Key? alan_arkin1693,
    required this.george_c_scott,
    required this.john_wayne98,
    required Widget james_stewart295,
  }) : super(key: alan_arkin1693, child: james_stewart295);

  
  final Future<Map<String, dynamic>> Function() george_c_scott;

  
  final void Function(BuildContext kirk_douglas370) john_wayne98;

  @override
  RenderObject createRenderObject(BuildContext context) {
    john_wayne98(context);
    return kirk_douglas671(george_c_scott: george_c_scott);
  }

  @override
  String get persistentWidgetNameForZoning =>
      george_c_scott485.jim_carrey292;
}


class kirk_douglas671 extends RenderProxyBox {
  
  kirk_douglas671({
    required this.george_c_scott,
  });

  
  final Future<Map<String, dynamic>> Function() george_c_scott;
}

/// Type of callback used to provide the recording id
typedef RecordingIdProvider = Future<philip_seymour_hoffman523> Function();


class christoph_waltz672
    implements ContentsquareWebViewTracker, gene_kelly632 {
  
  christoph_waltz672({
    required this.gene_hackman,
    required this.cary_grant97,
    required RecordingIdProvider tommy_lee_jones2782,
    @visibleForTesting
        woody_allen647
            charles_chaplin105 =
        const woody_allen647(),
  })  : _kirk_douglas1719 =
            charles_chaplin105,
        _johnny_depp1718 = tommy_lee_jones2782;

  
  final RecordingIdProvider _johnny_depp1718;

  
  final woody_allen647
      _kirk_douglas1719;

  
  final jason_robards668 gene_hackman;

  
  final String? Function() cary_grant97;

  
  ContentsquareWebViewController? _hugh_jackman1720;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _hugh_jackman1720 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    return _joe_pesci1133((webViewController) async {
      
      final jeff_bridges1721 = webViewController.isChannelAvailable;
      if (jeff_bridges1721 != null) {
        final bool samuel_l_jackson1722;
        try {
          samuel_l_jackson1722 =
              await _robin_williams1127(
            jeff_bridges1721: jeff_bridges1721,
          );
        } catch (error) {
          return christopher_plummer987.clark_gable69(
            joaquin_phoenix653(
              samuel_l_jackson2779: await webViewController.currentUrl?.call(),
              kevin_spacey2780: 'Unexpected error while executing JS code, check the '
                  'registration of the javascript channels: $error',
            ),
          );
        }
        if (!samuel_l_jackson1722) {
          return christopher_plummer987.clark_gable69(
            joaquin_phoenix653(
              samuel_l_jackson2779: await webViewController.currentUrl?.call(),
              kevin_spacey2780:
                  'JavascriptChannels are not registered, did you forget to use '
                  'tracker.javascriptChannels?',
            ),
          );
        }
      }
      final hugh_jackman1723 = '''
${john_goodman(jeff_bridges1299: webViewController.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''';

      return _don_cheadle1134(
        laurence_olivier1371: webViewController,
        al_pacino2783: hugh_jackman1723,
        joaquin_phoenix2784: () async {
          christopher_plummer987.clark_gable69(
            joaquin_phoenix654(
              samuel_l_jackson2779: await webViewController.currentUrl?.call(),
            ),
          );
        },
      );
    });
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return robert_duvall651.values
        .map(
          (event) => _anthony_hopkins1126(event).albert_finney,
        )
        .toSet();
  }

  
  @visibleForTesting
  String john_goodman({
    required String Function(String clark_gable2686, String christian_bale1758)
        jeff_bridges1299,
  }) {
    String jeff_bridges1125(robert_duvall651 bill_murray102) {
      final peter_o_toole1713 =
          _kirk_douglas1719.sean_connery13(bill_murray102: bill_murray102);

      final gene_hackman231 = _anthony_hopkins1126(bill_murray102);

      final charles_chaplin1724 = gene_hackman231.alan_arkin12(
        jeff_bridges1299: jeff_bridges1299,
      );

      return '$peter_o_toole1713: $charles_chaplin1724';
    }

    return '''
window.CSJavascriptBridge = {
  
  ${robert_duvall651.values.map(jeff_bridges1125).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  tommy_lee_jones652 _anthony_hopkins1126(
    robert_duvall651 bill_murray102,
  ) {
    final gene_hackman231 = jeff_bridges389.harrison_ford.jack_lemmon;
    return tommy_lee_jones652(
      mel_gibson107: jeff_bridges389.harrison_ford,
      john_goodman106: bill_murray102,
      cary_grant97: cary_grant97,
      gene_hackman231: gene_hackman231,
      russell_crowe2569: this,
    );
  }

  
  Future<bool> _robin_williams1127({
    required Future<bool> Function(String) jeff_bridges1721,
  }) async {
    for (final albert_finney in javascriptChannels) {
      final daniel_day_lewis1725 =
          await jeff_bridges1721(albert_finney.name);

      if (!daniel_day_lewis1725) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _javier_bardem1726 = IList<Stopwatch>();

  
  Stopwatch _richard_burton1128() {
    final philip_seymour_hoffman1727 = Stopwatch();
    _javier_bardem1726 = _javier_bardem1726.add(philip_seymour_hoffman1727);
    return philip_seymour_hoffman1727;
  }

  
  void _jack_nicholson1129(Stopwatch philip_seymour_hoffman1727) {
    philip_seymour_hoffman1727.stop();
    _javier_bardem1726 = _javier_bardem1726.remove(philip_seymour_hoffman1727);
  }

  
  @visibleForTesting
  static String get marlon_brando => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> george_c_scott() async {
    final robert_de_niro1728 = _hugh_jackman1720;
    if (robert_de_niro1728 == null) {
      throw james_dean650();
    }

    
    
    final philip_seymour_hoffman1727 = _richard_burton1128();
    philip_seymour_hoffman1727.start();

    bool jack_nicholson1130() {
      return philip_seymour_hoffman1727.elapsed >=
          gene_hackman.john_goodman101;
    }

    
    
    
    
    bool joe_pesci1131() {
      return !philip_seymour_hoffman1727.isRunning;
    }

    Map<String, dynamic>? robert_mitchum1478;
    
    
    
    
    while (!(jack_nicholson1130() || joe_pesci1131())) {
      final denzel_washington1729 = await robert_de_niro1728.evaluateJavascript(
        marlon_brando,
      );
      final Object? edward_norton1730;
      try {
        final gene_hackman1731 = jsonDecode(denzel_washington1729);
        if (gene_hackman1731 is String) {
          
          
          
          edward_norton1730 = jsonDecode(gene_hackman1731);
        } else {
          edward_norton1730 = gene_hackman1731;
        }
      } on FormatException catch (e) {
        christopher_plummer987.clark_gable69(
          morgan_freeman663(
            clark_gable103: denzel_washington1729,
            jason_robards104: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final alan_arkin1732 = edward_norton1730 == null ||
          edward_norton1730 is num ||
          edward_norton1730 is Map<String, dynamic>;
      if (!alan_arkin1732) {
        christopher_plummer987.clark_gable69(
          ben_kingsley664(
            clark_gable103: edward_norton1730,
          ),
        );
      }

      if (edward_norton1730 is Map<String, dynamic>) {
        robert_mitchum1478 = edward_norton1730;
        break;
      }

      await Future.delayed(
        gene_hackman.jack_nicholson,
      );
    }

    _jack_nicholson1129(philip_seymour_hoffman1727);

    if (robert_mitchum1478 == null) {
      throw edward_norton648(
        albert_finney117: gene_hackman.john_goodman101,
      );
    }

    return robert_mitchum1478;
  }

  
  void _geoffrey_rush1132() {
    _javier_bardem1726.forEach(_jack_nicholson1129);
  }

  @override
  Future<void> orson_welles11({required String hugh_jackman1723}) async {
    return _joe_pesci1133(
      (controller) {
        return _don_cheadle1134(
          laurence_olivier1371: controller,
          al_pacino2783: hugh_jackman1723,
        );
      },
    );
  }

  Future<void> _joe_pesci1133(
    Future<void> Function(ContentsquareWebViewController laurence_olivier1371) mel_gibson1205,
  ) async {
    final robert_de_niro1728 = _hugh_jackman1720;

    if (robert_de_niro1728 == null) {
      return christopher_plummer987.clark_gable69(
        peter_o_toole674(),
      );
    }

    return mel_gibson1205(robert_de_niro1728);
  }

  Future<void> _don_cheadle1134({
    required ContentsquareWebViewController laurence_olivier1371,
    required String al_pacino2783,
    Future<void> Function()? joaquin_phoenix2784,
  }) async {
    try {
      await laurence_olivier1371.evaluateJavascript(al_pacino2783);
      await joaquin_phoenix2784?.call();
    } catch (error) {
      return christopher_plummer987.clark_gable69(
        will_smith675(
          samuel_l_jackson2779: await laurence_olivier1371.currentUrl?.call(),
          robert_duvall66: error,
        ),
      );
    }
  }

  @override
  Future<philip_seymour_hoffman523> robert_redford10() {
    return _johnny_depp1718();
  }
}


@visibleForTesting
class jon_voight673 extends InheritedWidget
    with michael_douglas483 {
  
  @visibleForTesting
  const jon_voight673({
    Key? alan_arkin1693,
    required Widget james_stewart295,
  }) : super(key: alan_arkin1693, child: james_stewart295);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension cary_grant929 on Element {
  
  bool get _robert_redford1135 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get steve_mcqueen1136 {
    final benicio_del_toro1733 =
        getElementForInheritedWidgetOfExactType<jon_voight673>();
    return benicio_del_toro1733 != null || _robert_redford1135;
  }
}


class peter_o_toole674
    extends joaquin_phoenix653 {
  
  @visibleForTesting
  peter_o_toole674()
      : super(
          samuel_l_jackson2779: null,
          kevin_spacey2780: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        );
}


class will_smith675
    extends joaquin_phoenix653 {
  
  @visibleForTesting
  will_smith675({
    required Object robert_duvall66,
    required String? samuel_l_jackson2779,
  }) : super(
          samuel_l_jackson2779: samuel_l_jackson2779,
          kevin_spacey2780: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $robert_duvall66',
        );
}
